﻿namespace SuperName
{
    public class SuperName : TabsStudioExt.ITabsStudioAddin
    {
        public SuperName()
        {
            superGroupProperties = new TabsStudioExt.Util.SuperGroupProperties();
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            engine.TabCreated += OnTabCreated;
            engine.TabDestroyed += OnTabDestroyed;

            firstTabPropertyMonitor = new IsFirstTabPropertyMonitor();
            firstTabPropertyMonitor.OnFirstTabPropertyChanged += OnFirstTabPropertyChanged;
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
            engine.TabDestroyed -= OnTabDestroyed;

            firstTabPropertyMonitor.OnFirstTabPropertyChanged -= OnFirstTabPropertyChanged;
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            SetName(e.Tab);
            firstTabPropertyMonitor.OnTabCreated(e);
        }

        private void OnTabDestroyed(object sender, TabsStudioExt.TabEventArgs e)
        {
            firstTabPropertyMonitor.OnTabDestroyed(e);
        }

        private void SetName(TabsStudioExt.Tab tab)
        {
            RemoveSuperNameTag(tab);
            if (superGroupProperties.IsFirstTab(tab.TabItem))
            {
                string name = GetProjectName(tab);
                if (name != null)
                    AddSuperNameTag(tab, name);
            }
        }

        private void AddSuperNameTag(TabsStudioExt.Tab tab, string name)
        {
            tab.TitleManager.Transforms.Add(
                new TabsStudioExt.TitleTransform("^.*$", "[" + name + "] " + "$_", superNameTag));
            tab.TitleManager.ApplyTransforms();
        }

        private void RemoveSuperNameTag(TabsStudioExt.Tab tab)
        {
            foreach (TabsStudioExt.TitleTransform t in tab.TitleManager.Transforms)
            {
                if (t.Id == superNameTag)
                {
                    tab.TitleManager.Transforms.Remove(t);
                    tab.TitleManager.ApplyTransforms();
                    return;
                }
            }
        }

        private string GetProjectName(TabsStudioExt.Tab tab)
        {
            EnvDTE.Window w = tab.TabExtensions[0].Window;
            try
            {
                if (w.Document != null && w.Document.ProjectItem != null &&
                    w.Document.ProjectItem.ContainingProject != null)
                {
                    string projectName = w.Document.ProjectItem.ContainingProject.Name;
                    if (System.IO.Path.IsPathRooted(projectName))
                        projectName = System.IO.Path.GetFileName(System.IO.Path.GetDirectoryName(projectName));
                    return projectName;
                }
            }
            catch (System.Exception)
            {
            }
            return null;
        }

        private void OnFirstTabPropertyChanged(TabsStudioExt.Tab tab)
        {
            SetName(tab);
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private TabsStudioExt.Util.SuperGroupProperties superGroupProperties;
        private IsFirstTabPropertyMonitor firstTabPropertyMonitor;
        private static string superNameTag = "[SuperName]";
    }
}